/*
 * RTC8564 RTCCu Version 1.00
 * File:        rtc.c
 * Target:      PIC18F14K50 48MHz
 * RpC:   Microchip MPLAB XC8 C Compiler V1.20
 * :     PICKit 3
 * [X:     2013/09/29 15:00
 *
 * AdቺtOoARTCǂݏAC^[o^C}[On/Off
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <p18f14k50.h>
#include <plib\delays.h>
#include <plib\i2c.h>
#include "i2c_ex.h"
#include "rtc.h"

//  RTC̏l

unsigned char rtc_init_data[]={
   0b00100000, 0b00000000, // CTL1,CTL2
   0x00,0x40,       // 02h b,
   0x14,0x22,       // 04h ,
   0x00,0x09,       // 06h j,
   0x13,0x80,       // 08h N,A[ 
   0x80,0x80,       // 0ah A[ ,A[  
   0x80,0x00,       // 0ch A[ j,CLOCKOUT g
   0x00,0x00};      // 0eh ^C}[CTL,^C}[

//  RTC

signed char rtc_init(void){
    unsigned char* buf=rtc_init_data;
    unsigned char n;
    signed char ret;

    do{
        SSPCON1bits.WCOL=0;
        IdleI2C();
        StartI2C();
        while(SSPCON2bits.SEN);
        if(PIR2bits.BCLIF)
            return -10;
    }while(SSPCON1bits.WCOL);

    if(ret=i2c_write(RTC8564_SADS) )
        return ret-2;

    if(ret=i2c_write(RTC_CTL1))
        return ret-4;
    for(n=0;n<16;n++)
        if(ret=i2c_write(buf[n]))
            return ret;
    StopI2C();
    while(SSPCON2bits.PEN);
    return 0;
}

//  \

void rtc_sput(unsigned char* buf,struct RTC_DATE* d){
    rtc_date_sput(buf,d);
    buf[8]='\n';
    rtc_time_sput(buf+9,d);
}

//  RTCJEgJn

signed char rtc_start(void){
    signed char ret;
    do{
        SSPCON1bits.WCOL=0;
        IdleI2C();
        StartI2C();
        while(SSPCON2bits.SEN);
        if(PIR2bits.BCLIF)
            return -10;
    }while(SSPCON1bits.WCOL);

    if( ret=i2c_write(RTC8564_SADS) )
        return ret-2;

    if(ret=i2c_write(RTC_CTL1))
        return ret-4;
    if(ret=i2c_write(0))
        return ret-6;
    StopI2C();
    while(SSPCON2bits.PEN);
    return 0;
}

//  RTCJEg~

signed char rtc_stop(void){
    signed char ret;
    do{
        SSPCON1bits.WCOL=0;
        IdleI2C();
        StartI2C();
        while(SSPCON2bits.SEN);
        if(PIR2bits.BCLIF)
            return -10;
    }while(SSPCON1bits.WCOL);
    if(ret=i2c_write(RTC8564_SADS))
        return ret-2;
    if(ret=i2c_write(RTC_CTL1))
        return ret-4;
    if(ret=i2c_write(0b00100000))
        return ret-6;
    StopI2C();
    while(SSPCON2bits.PEN);
    return 0;
}

//  ݒ

signed char rtc_write(struct RTC_DATE* d){
    int n;
    d->month &= 0x1f;
    d->week &= 0x07;
    d->day &= 0x3f;
    d->hour &= 0x3f;
    d->min &= 0x7f;
    d->sec &= 0x7f;
    unsigned char* buf=(unsigned char*)d;
    signed char ret;
    do{
        SSPCON1bits.WCOL=0;
        IdleI2C();
        StartI2C();
        while(SSPCON2bits.SEN);
        if(PIR2bits.BCLIF)
            return -10;
    }while(SSPCON1bits.WCOL);
    if(ret=i2c_write(RTC8564_SADS))
        return ret-2;
    if(ret=i2c_write(RTC_SEC))
        return ret;
    for(n=0;n<7;n++)
        if(ret=i2c_write(buf[n]))
            return ret-4;
    StopI2C();
    while(SSPCON2bits.PEN);
    return 0;

}

//  ǂݏo

signed char _rtc_read(struct RTC_DATE* d){
    unsigned char n;
    unsigned char* buf=(unsigned char*)d;
    signed char ret;

    do{
        SSPCON1bits.WCOL=0;
        IdleI2C();
        StartI2C();
        while(SSPCON2bits.SEN);
        if(PIR2bits.BCLIF){
            return -10;
        }
    }while(SSPCON1bits.WCOL);

    ret=i2c_write(RTC8564_SADS);
    if(ret)
        return ret-2;
    if(ret=i2c_write(RTC_SEC))
        return ret-4;
    RestartI2C();
    while(SSPCON2bits.RSEN);

    if(ret=i2c_write(RTC8564_SADS | 1))
        return ret-6;

    for(n=0;n<7;n++){
        buf[n]=ReadI2C();
        SSPCON2bits.ACKDT=0;
        SSPCON2bits.ACKEN=1;
        while(SSPCON2bits.ACKEN);
    }
    buf[n]=ReadI2C();
    SSPCON2bits.ACKDT=1;
    SSPCON2bits.ACKEN=1;
    while(SSPCON2bits.ACKEN);
    StopI2C();
    while(SSPCON2bits.PEN);

    d->month = d->month & 0x1f;
    d->week = d->week & 0x07;
    d->day = d->day & 0x3f;
    d->hour = d->hour & 0x3f;
    d->min = d->min & 0x7f;
    d->sec = d->sec & 0x7f;
    return 0;
}

//  ǂݏo

signed char rtc_read(struct RTC_DATE* d){
    unsigned char try=TRY;
    signed char ret;
    do{
        if( (ret=_rtc_read(d)) == 0 )
            break;
        Delay1TCYx( 24);  //  2s*(48MHz/4)= 24
    }while(--try);
    return ret;
}

//  VLrbg`FbN  1:Kv 0:sv

signed char rtc_vl_chk(void){
    unsigned char n;
    unsigned char sec;
    signed char ret;
    do{
        SSPCON1bits.WCOL=0;
        IdleI2C();
        StartI2C();
        if(PIR2bits.BCLIF){
           return -12;
       }
        while(SSPCON2bits.SEN);

    }while(SSPCON1bits.WCOL);
    ret=i2c_write(RTC8564_SADS);

    if(i2c_write(RTC_SEC))
        return 1;
    RestartI2C();
    while(SSPCON2bits.RSEN);
    if(i2c_write(RTC8564_SADS | 1))
        return 1;
    while(DataRdyI2C());
    sec=ReadI2C();
    SSPCON2bits.ACKDT=1;
    SSPCON2bits.ACKEN=1;
    while(SSPCON2bits.ACKEN);
    StopI2C();
    while(SSPCON2bits.PEN);
    if(sec & 0x80)
        return 1;
    else
        return 0;
}

//  1byteǂݏo

signed char rtc_read_byte(unsigned char ads){
    unsigned char data;
    signed char ret;
    do{
        SSPCON1bits.WCOL=0;
        IdleI2C();
        StartI2C();
        if(PIR2bits.BCLIF){
           return -12;
       }
        while(SSPCON2bits.SEN);

    }while(SSPCON1bits.WCOL);
    ret=i2c_write(RTC8564_SADS);

    if(i2c_write(ads))
        return 1;
    RestartI2C();
    while(SSPCON2bits.RSEN);
    if(i2c_write(RTC8564_SADS | 1))
        return 1;
    while(DataRdyI2C());
    data=ReadI2C();
    SSPCON2bits.ACKDT=1;
    SSPCON2bits.ACKEN=1;
    while(SSPCON2bits.ACKEN);
    StopI2C();
    while(SSPCON2bits.PEN);
    return data;
}

//  1byteo

signed char rtc_write_byte(unsigned char ads,unsigned char data){
    signed char ret;
    do{
        SSPCON1bits.WCOL=0;
        IdleI2C();
        StartI2C();
        while(SSPCON2bits.SEN);
        if(PIR2bits.BCLIF)
            return -10;
    }while(SSPCON1bits.WCOL);

    if( ret=i2c_write(RTC8564_SADS) )
        return ret-2;

    if(ret=i2c_write(ads))
        return ret-4;
    if(ret=i2c_write(data))
        return ret-6;
    StopI2C();
    while(SSPCON2bits.PEN);
    return 0;
}

//  1b^C}[Jn

void rtc_timer_on(void){
    rtc_write_byte(RTC_TIMERCTL,0); //  ^C}[OFF TE=0
    rtc_write_byte(RTC_CTL2,0); // TF=TIE=0

    rtc_write_byte(RTC_CTL2,0b00010001);    // TI/TP=1 荞ݓp,TIE=1 INTo̓Clu
    rtc_write_byte(RTC_TIMERCTL,0b00000010); // 1Hz TD=10
    rtc_write_byte(RTC_TIMER,0x1);  // 1buɊ荞
    rtc_write_byte(RTC_TIMERCTL,0b10000010); // ^C}[ON TE=1
}

//  1b^C}[~

void rtc_timer_off(void){
    rtc_write_byte(RTC_TIMERCTL,0); //  ^C}[OFF@TE=0
    rtc_write_byte(RTC_CTL2,0);
}
